@@ -41,13 +41,10 @@
 #include "up-types.h"
 #include "up-device-idevice.h"
 
-#define UP_DEVICE_IDEVICE_DEFAULT_POLL_TIME	60 /* seconds */
-
 struct UpDeviceIdevicePrivate
 {
 	idevice_t		 dev;
 	lockdownd_client_t	 client;
-	guint			 poll_timer_id;
 };
 
 G_DEFINE_TYPE (UpDeviceIdevice, up_device_idevice, UP_TYPE_DEVICE)
@@ -82,8 +79,6 @@ up_device_idevice_coldplug (UpDevice *device)
 	GUdevDevice *native;
 	const gchar *uuid;
 	const gchar *model;
-	plist_t dict, node;
-	guint64 poll_seconds = 0;
 	idevice_t dev = NULL;
 	lockdownd_client_t client = NULL;
 	UpDeviceKind kind;
@@ -105,17 +100,6 @@ up_device_idevice_coldplug (UpDevice *device)
 	if (LOCKDOWN_E_SUCCESS != lockdownd_client_new_with_handshake (dev, &client, "upower"))
 		goto out;
 
-	/* Get the poll timeout */
-	if (lockdownd_get_value (client, "com.apple.mobile.iTunes", "BatteryPollInterval", &dict) != LOCKDOWN_E_SUCCESS)
-		goto out;
-
-	node = plist_dict_get_item (dict, "BatteryPollInterval");
-	if (node != NULL)
-		plist_get_uint_val (node, &poll_seconds);
-	if (poll_seconds == 0)
-		poll_seconds = UP_DEVICE_IDEVICE_DEFAULT_POLL_TIME;
-	plist_free (dict);
-
 	/* Set up struct */
 	idevice->priv->dev = dev;
 	idevice->priv->client = client;
@@ -150,12 +134,7 @@ up_device_idevice_coldplug (UpDevice *device)
 	idevice->priv->client = NULL;
 
 	/* set up a poll */
-	idevice->priv->poll_timer_id = g_timeout_add_seconds (poll_seconds,
-							      (GSourceFunc) up_device_idevice_poll_cb, idevice);
-
-#if GLIB_CHECK_VERSION(2,25,8)
-	g_source_set_name_by_id (idevice->priv->poll_timer_id, "[UpDeviceIdevice] poll");
-#endif
+	up_daemon_start_poll (G_OBJECT (idevice), (GSourceFunc) up_device_idevice_poll_cb);
 	return TRUE;
 
 out:
@@ -258,8 +237,7 @@ up_device_idevice_finalize (GObject *object)
 	idevice = UP_DEVICE_IDEVICE (object);
 	g_return_if_fail (idevice->priv != NULL);
 
-	if (idevice->priv->poll_timer_id > 0)
-		g_source_remove (idevice->priv->poll_timer_id);
+	up_daemon_stop_poll (object);
 	if (idevice->priv->client != NULL)
 		lockdownd_client_free (idevice->priv->client);
 	idevice_free (idevice->priv->dev);
