@@ -22,7 +22,6 @@
 #define __UP_DAEMON_H__
 
 #include <glib-object.h>
-#include <polkit/polkit.h>
 #include <dbus/dbus-glib.h>
 
 #include "up-types.h"
@@ -77,8 +76,6 @@ void		 up_daemon_set_lid_is_closed	(UpDaemon		*daemon,
 						 gboolean		 lid_is_closed);
 void		 up_daemon_set_lid_is_present	(UpDaemon		*daemon,
 						 gboolean		 lid_is_present);
-void		 up_daemon_set_is_docked	(UpDaemon		*daemon,
-						 gboolean		 is_docked);
 void		 up_daemon_set_on_battery	(UpDaemon		*daemon,
 						 gboolean		 on_battery);
 void		 up_daemon_set_warning_level	(UpDaemon		*daemon,
