@@ -44,12 +44,14 @@ struct UpDevicePrivate
 {
 	gchar			*object_path;
 	DBusGConnection		*system_bus_connection;
-	DBusGProxy		*system_bus_proxy;
 	UpDaemon		*daemon;
 	UpHistory		*history;
 	GObject			*native;
 	gboolean		 has_ever_refresh;
-	gboolean		 during_coldplug;
+
+	/* PropertiesChanged to be emitted */
+	GHashTable		*changed_props;
+	guint			 props_idle_id;
 
 	/* properties */
 	guint64			 update_time;
@@ -78,9 +80,8 @@ struct UpDevicePrivate
 	gint64			 time_to_full;		/* seconds */
 	gdouble			 percentage;		/* percent */
 	gdouble			 temperature;		/* degrees C */
-	gboolean		 recall_notice;
-	gchar			*recall_vendor;
-	gchar			*recall_url;
+	UpDeviceLevel		 warning_level;		/* computed */
+	const gchar		*icon_name;		/* computed */
 };
 
 static gboolean	up_device_register_device	(UpDevice *device);
@@ -113,19 +114,11 @@ enum {
 	PROP_PERCENTAGE,
 	PROP_TEMPERATURE,
 	PROP_TECHNOLOGY,
-	PROP_RECALL_NOTICE,
-	PROP_RECALL_VENDOR,
-	PROP_RECALL_URL,
+	PROP_WARNING_LEVEL,
+	PROP_ICON_NAME,
 	PROP_LAST
 };
 
-enum {
-	SIGNAL_CHANGED,
-	SIGNAL_LAST,
-};
-
-static guint signals[SIGNAL_LAST] = { 0 };
-
 G_DEFINE_TYPE (UpDevice, up_device, G_TYPE_OBJECT)
 #define UP_DEVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), UP_TYPE_DEVICE, UpDevicePrivate))
 #define UP_DBUS_STRUCT_UINT_DOUBLE_UINT (dbus_g_type_get_struct ("GValueArray", \
@@ -133,6 +126,12 @@ G_DEFINE_TYPE (UpDevice, up_device, G_TYPE_OBJECT)
 #define UP_DBUS_STRUCT_DOUBLE_DOUBLE (dbus_g_type_get_struct ("GValueArray", \
 	G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_INVALID))
 
+#define UP_DEVICES_DBUS_PATH "/org/freedesktop/UPower/devices"
+
+static void up_device_queue_changed_property (UpDevice    *device,
+					      const gchar *property,
+					      GVariant    *value);
+
 /**
  * up_device_error_quark:
  **/
@@ -171,6 +170,153 @@ up_device_error_get_type (void)
 	return etype;
 }
 
+/* This needs to be called when one of those properties changes:
+ * state
+ * power_supply
+ * percentage
+ * time_to_empty
+ *
+ * type should not change for non-display devices
+ */
+static void
+update_warning_level (UpDevice *device)
+{
+	UpDeviceLevel warning_level;
+
+	/* Not finished setting up the object? */
+	if (device->priv->daemon == NULL)
+		return;
+
+	warning_level = up_daemon_compute_warning_level (device->priv->daemon,
+							 device->priv->state,
+							 device->priv->type,
+							 device->priv->power_supply,
+							 device->priv->percentage,
+							 device->priv->time_to_empty);
+
+	if (warning_level == device->priv->warning_level)
+		return;
+
+	device->priv->warning_level = warning_level;
+	g_object_notify (G_OBJECT (device), "warning-level");
+
+	up_device_queue_changed_property (device, "warning-level", g_variant_new_uint32 (device->priv->warning_level));
+}
+
+static const gchar *
+get_device_charge_icon (gdouble  percentage,
+			gboolean charging)
+{
+	if (percentage < 10)
+		return charging ? "battery-caution-charging-symbolic" : "battery-caution-symbolic";
+	else if (percentage < 30)
+		return charging ? "battery-low-charging-symbolic" : "battery-low-symbolic";
+	else if (percentage < 60)
+		return charging ? "battery-good-charging-symbolic" : "battery-good-symbolic";
+	return charging ? "battery-full-charging-symbolic" : "battery-full-symbolic";
+}
+
+/* This needs to be called when one of those properties changes:
+ * type
+ * state
+ * percentage
+ * is-present
+ */
+static void
+update_icon_name (UpDevice *device)
+{
+	const gchar *icon_name = NULL;
+
+	/* get the icon from some simple rules */
+	if (device->priv->type == UP_DEVICE_KIND_LINE_POWER) {
+		icon_name = "ac-adapter-symbolic";
+	} else {
+
+		if (!device->priv->is_present) {
+			icon_name = "battery-missing-symbolic";
+
+		} else {
+			switch (device->priv->state) {
+			case UP_DEVICE_STATE_EMPTY:
+				icon_name = "battery-empty-symbolic";
+				break;
+			case UP_DEVICE_STATE_FULLY_CHARGED:
+				icon_name = "battery-full-charged-symbolic";
+				break;
+			case UP_DEVICE_STATE_CHARGING:
+			case UP_DEVICE_STATE_PENDING_CHARGE:
+				icon_name = get_device_charge_icon (device->priv->percentage, TRUE);
+				break;
+			case UP_DEVICE_STATE_DISCHARGING:
+			case UP_DEVICE_STATE_PENDING_DISCHARGE:
+				icon_name = get_device_charge_icon (device->priv->percentage, FALSE);
+				break;
+			default:
+				icon_name = "battery-missing-symbolic";
+			}
+		}
+	}
+
+	if (g_strcmp0 (icon_name, device->priv->icon_name) == 0)
+		return;
+
+	device->priv->icon_name = icon_name;
+	g_object_notify (G_OBJECT (device), "icon-name");
+
+	up_device_queue_changed_property (device, "icon-name", g_variant_new_string (device->priv->icon_name));
+}
+
+static gboolean
+changed_props_idle_cb (gpointer user_data)
+{
+	UpDevice *device = user_data;
+
+	/* D-Bus */
+	up_daemon_emit_properties_changed (device->priv->system_bus_connection,
+					   device->priv->object_path,
+					   "org.freedesktop.UPower.Device",
+					   device->priv->changed_props);
+	g_clear_pointer (&device->priv->changed_props, g_hash_table_unref);
+	device->priv->props_idle_id = 0;
+
+	return G_SOURCE_REMOVE;
+}
+
+/**
+ * up_device_queue_changed_property:
+ **/
+static void
+up_device_queue_changed_property (UpDevice    *device,
+				  const gchar *property,
+				  GVariant    *value)
+{
+	gchar *dbus_prop;
+	gchar **items;
+	guint i;
+
+	g_return_if_fail (UP_IS_DEVICE (device));
+
+	if (device->priv->system_bus_connection == NULL)
+		return;
+
+	if (!device->priv->changed_props) {
+		device->priv->changed_props = g_hash_table_new_full (g_str_hash, g_str_equal,
+								     g_free, (GDestroyNotify) g_variant_unref);
+	}
+
+	items = g_strsplit (property, "-", -1);
+	for (i = 0; items[i] != NULL; i++)
+		*items[i] = g_ascii_toupper (*items[i]);
+	dbus_prop = g_strjoinv (NULL, items);
+	g_strfreev (items);
+
+	g_hash_table_insert (device->priv->changed_props,
+			     dbus_prop, value);
+
+	if (device->priv->props_idle_id == 0)
+		device->priv->props_idle_id = g_idle_add (changed_props_idle_cb, device);
+}
+
 /**
  * up_device_get_property:
  **/
@@ -257,14 +403,11 @@ up_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 	case PROP_TECHNOLOGY:
 		g_value_set_uint (value, device->priv->technology);
 		break;
-	case PROP_RECALL_NOTICE:
-		g_value_set_boolean (value, device->priv->recall_notice);
+	case PROP_WARNING_LEVEL:
+		g_value_set_uint (value, device->priv->warning_level);
 		break;
-	case PROP_RECALL_VENDOR:
-		g_value_set_string (value, device->priv->recall_vendor);
-		break;
-	case PROP_RECALL_URL:
-		g_value_set_string (value, device->priv->recall_url);
+	case PROP_ICON_NAME:
+		g_value_set_string (value, device->priv->icon_name);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -302,15 +445,18 @@ up_device_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 		break;
 	case PROP_TYPE:
 		device->priv->type = g_value_get_uint (value);
+		update_icon_name (device);
 		break;
 	case PROP_POWER_SUPPLY:
 		device->priv->power_supply = g_value_get_boolean (value);
+		update_warning_level (device);
 		break;
 	case PROP_ONLINE:
 		device->priv->online = g_value_get_boolean (value);
 		break;
 	case PROP_IS_PRESENT:
 		device->priv->is_present = g_value_get_boolean (value);
+		update_icon_name (device);
 		break;
 	case PROP_IS_RECHARGEABLE:
 		device->priv->is_rechargeable = g_value_get_boolean (value);
@@ -323,6 +469,8 @@ up_device_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 		break;
 	case PROP_STATE:
 		device->priv->state = g_value_get_uint (value);
+		update_warning_level (device);
+		update_icon_name (device);
 		break;
 	case PROP_CAPACITY:
 		device->priv->capacity = g_value_get_double (value);
@@ -350,12 +498,15 @@ up_device_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 		break;
 	case PROP_TIME_TO_EMPTY:
 		device->priv->time_to_empty = g_value_get_int64 (value);
+		update_warning_level (device);
 		break;
 	case PROP_TIME_TO_FULL:
 		device->priv->time_to_full = g_value_get_int64 (value);
 		break;
 	case PROP_PERCENTAGE:
 		device->priv->percentage = g_value_get_double (value);
+		update_warning_level (device);
+		update_icon_name (device);
 		break;
 	case PROP_TEMPERATURE:
 		device->priv->temperature = g_value_get_double (value);
@@ -363,21 +514,19 @@ up_device_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 	case PROP_TECHNOLOGY:
 		device->priv->technology = g_value_get_uint (value);
 		break;
-	case PROP_RECALL_NOTICE:
-		device->priv->recall_notice = g_value_get_boolean (value);
-		break;
-	case PROP_RECALL_VENDOR:
-		g_free (device->priv->recall_vendor);
-		device->priv->recall_vendor = g_strdup (g_value_get_string (value));
-		break;
-	case PROP_RECALL_URL:
-		g_free (device->priv->recall_url);
-		device->priv->recall_url = g_strdup (g_value_get_string (value));
+	case PROP_WARNING_LEVEL:
+		device->priv->warning_level = g_value_get_uint (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
+		return;
 	}
+
+	if (G_VALUE_TYPE (value) == G_TYPE_STRING &&
+	    g_value_get_string (value) == NULL)
+		up_device_queue_changed_property (device, pspec->name, g_variant_new_string (""));
+	else
+		up_device_queue_changed_property (device, pspec->name, dbus_g_value_build_g_variant (value));
 }
 
 /**
@@ -399,25 +548,6 @@ up_device_get_on_battery (UpDevice *device, gboolean *on_battery)
 	return klass->get_on_battery (device, on_battery);
 }
 
-/**
- * up_device_get_low_battery:
- *
- * Note: Only implement for system devices, i.e. ones supplying the system
- **/
-gboolean
-up_device_get_low_battery (UpDevice *device, gboolean *low_battery)
-{
-	UpDeviceClass *klass = UP_DEVICE_GET_CLASS (device);
-
-	g_return_val_if_fail (UP_IS_DEVICE (device), FALSE);
-
-	/* no support */
-	if (klass->get_low_battery == NULL)
-		return FALSE;
-
-	return klass->get_low_battery (device, low_battery);
-}
-
 /**
  * up_device_get_online:
  *
@@ -555,50 +685,58 @@ up_device_coldplug (UpDevice *device, UpDaemon *daemon, GObject *native)
 	native_path = up_native_get_native_path (native);
 	device->priv->native_path = g_strdup (native_path);
 
-	/* stop signals and callbacks */
-	g_object_freeze_notify (G_OBJECT(device));
-	device->priv->during_coldplug = TRUE;
-
 	/* coldplug source */
 	if (klass->coldplug != NULL) {
 		ret = klass->coldplug (device);
 		if (!ret) {
 			g_debug ("failed to coldplug %s", device->priv->native_path);
-			goto out;
+			goto bail;
 		}
 	}
 
-	/* only put on the bus if we succeeded */
-	ret = up_device_register_device (device);
-	if (!ret) {
-		g_warning ("failed to register device %s", device->priv->native_path);
-		goto out;
-	}
-
 	/* force a refresh, although failure isn't fatal */
 	ret = up_device_refresh_internal (device);
 	if (!ret) {
 		g_debug ("failed to refresh %s", device->priv->native_path);
 
-		/* TODO: refresh should really have seporate
+		/* TODO: refresh should really have separate
 		 *       success _and_ changed parameters */
-		ret = TRUE;
 		goto out;
 	}
 
 	/* get the id so we can load the old history */
 	id = up_device_get_id (device);
-	if (id != NULL)
+	if (id != NULL) {
 		up_history_set_id (device->priv->history, id);
-
+		g_free (id);
+	}
 out:
-	/* start signals and callbacks */
-	g_object_thaw_notify (G_OBJECT(device));
-	device->priv->during_coldplug = FALSE;
-	g_free (id);
+	/* only put on the bus if we succeeded */
+	ret = up_device_register_device (device);
+	if (!ret) {
+		g_warning ("failed to register device %s", device->priv->native_path);
+		goto out;
+	}
+bail:
 	return ret;
 }
 
+/**
+ * up_device_register_display_device:
+ **/
+gboolean
+up_device_register_display_device (UpDevice *device,
+				   UpDaemon *daemon)
+{
+	g_return_val_if_fail (UP_IS_DEVICE (device), FALSE);
+
+	device->priv->daemon = g_object_ref (daemon);
+	device->priv->object_path = g_build_filename (UP_DEVICES_DBUS_PATH, "DisplayDevice", NULL);
+	dbus_g_connection_register_g_object (device->priv->system_bus_connection,
+					     device->priv->object_path, G_OBJECT (device));
+	return TRUE;
+}
+
 /**
  * up_device_get_statistics:
  **/
@@ -825,7 +963,7 @@ up_device_compute_object_path (UpDevice *device)
 		if (id[i] == ':')
 			id[i] = 'o';
 	}
-	object_path = g_build_filename ("/org/freedesktop/UPower/devices", id, NULL);
+	object_path = g_build_filename (UP_DEVICES_DBUS_PATH, id, NULL);
 
 	g_free (basename);
 	g_free (id);
@@ -839,19 +977,11 @@ up_device_compute_object_path (UpDevice *device)
 static gboolean
 up_device_register_device (UpDevice *device)
 {
-	gboolean ret = TRUE;
-
 	device->priv->object_path = up_device_compute_object_path (device);
 	g_debug ("object path = %s", device->priv->object_path);
 	dbus_g_connection_register_g_object (device->priv->system_bus_connection,
 					     device->priv->object_path, G_OBJECT (device));
-	device->priv->system_bus_proxy = dbus_g_proxy_new_for_name (device->priv->system_bus_connection,
-								    DBUS_SERVICE_DBUS, DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS);
-	if (device->priv->system_bus_proxy == NULL) {
-		g_warning ("proxy invalid");
-		ret = FALSE;
-	}
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -862,21 +992,12 @@ up_device_perhaps_changed_cb (GObject *object, GParamSpec *pspec, UpDevice *devi
 {
 	g_return_if_fail (UP_IS_DEVICE (device));
 
-	/* don't proxy during coldplug */
-	if (device->priv->during_coldplug)
-		return;
-
 	/* save new history */
 	up_history_set_state (device->priv->history, device->priv->state);
 	up_history_set_charge_data (device->priv->history, device->priv->percentage);
 	up_history_set_rate_data (device->priv->history, device->priv->energy_rate);
 	up_history_set_time_full_data (device->priv->history, device->priv->time_to_full);
 	up_history_set_time_empty_data (device->priv->history, device->priv->time_to_empty);
-
-	/*  The order here matters; we want Device::Changed() before
-	 *  the DeviceChanged() signal on the main object */
-	g_debug ("emitting changed on %s", device->priv->native_path);
-	g_signal_emit (device, signals[SIGNAL_CHANGED], 0);
 }
 
 /**
@@ -888,13 +1009,6 @@ up_device_init (UpDevice *device)
 	GError *error = NULL;
 
 	device->priv = UP_DEVICE_GET_PRIVATE (device);
-	device->priv->object_path = NULL;
-	device->priv->system_bus_connection = NULL;
-	device->priv->system_bus_proxy = NULL;
-	device->priv->daemon = NULL;
-	device->priv->native = NULL;
-	device->priv->has_ever_refresh = FALSE;
-	device->priv->during_coldplug = FALSE;
 	device->priv->history = up_history_new ();
 
 	device->priv->system_bus_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
@@ -922,14 +1036,14 @@ up_device_finalize (GObject *object)
 		g_object_unref (device->priv->native);
 	if (device->priv->daemon != NULL)
 		g_object_unref (device->priv->daemon);
+	if (device->priv->props_idle_id != 0)
+		g_source_remove (device->priv->props_idle_id);
 	g_object_unref (device->priv->history);
 	g_free (device->priv->object_path);
 	g_free (device->priv->vendor);
 	g_free (device->priv->model);
 	g_free (device->priv->serial);
 	g_free (device->priv->native_path);
-	g_free (device->priv->recall_vendor);
-	g_free (device->priv->recall_url);
 
 	G_OBJECT_CLASS (up_device_parent_class)->finalize (object);
 }
@@ -947,14 +1061,6 @@ up_device_class_init (UpDeviceClass *klass)
 
 	g_type_class_add_private (klass, sizeof (UpDevicePrivate));
 
-	signals[SIGNAL_CHANGED] =
-		g_signal_new ("changed",
-			      G_OBJECT_CLASS_TYPE (klass),
-			      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-			      0, NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0);
-
 	dbus_g_object_type_install_info (UP_TYPE_DEVICE, &dbus_glib_up_device_object_info);
 
 	/**
@@ -1185,33 +1291,27 @@ up_device_class_init (UpDeviceClass *klass)
 					 g_param_spec_double ("temperature", NULL, NULL,
 							      0.0, G_MAXDOUBLE, 0.0,
 							      G_PARAM_READWRITE));
+
 	/**
-	 * UpDevice:recall-notice:
+	 * UpDevice:warning-level:
 	 */
 	g_object_class_install_property (object_class,
-					 PROP_RECALL_NOTICE,
-					 g_param_spec_boolean ("recall-notice",
-							       NULL, NULL,
-							       FALSE,
-							       G_PARAM_READWRITE));
+					 PROP_WARNING_LEVEL,
+					 g_param_spec_uint ("warning-level",
+							    NULL, NULL,
+							    UP_DEVICE_LEVEL_UNKNOWN,
+							    UP_DEVICE_LEVEL_LAST,
+							    UP_DEVICE_LEVEL_UNKNOWN,
+							    G_PARAM_READWRITE));
+
 	/**
-	 * UpDevice:recall-vendor:
+	 * UpDevice:icon:
 	 */
 	g_object_class_install_property (object_class,
-					 PROP_RECALL_VENDOR,
-					 g_param_spec_string ("recall-vendor",
-							      NULL, NULL,
-							      NULL,
-							      G_PARAM_READWRITE));
-	/**
-	 * UpDevice:recall-url:
-	 */
-	g_object_class_install_property (object_class,
-					 PROP_RECALL_URL,
-					 g_param_spec_string ("recall-url",
-							      NULL, NULL,
-							      NULL,
-							      G_PARAM_READWRITE));
+					 PROP_ICON_NAME,
+					 g_param_spec_string ("icon-name",
+							      NULL, NULL, NULL,
+							      G_PARAM_READABLE));
 
 	dbus_g_error_domain_register (UP_DEVICE_ERROR, NULL, UP_DEVICE_TYPE_ERROR);
 }
