@@ -123,7 +123,6 @@ up_client_print (UpClient *client)
 	UpDeviceLevel warning_level;
 	gboolean lid_is_closed;
 	gboolean lid_is_present;
-	gboolean is_docked;
 	char *action;
 
 	g_object_get (client,
@@ -131,14 +130,12 @@ up_client_print (UpClient *client)
 		      "on-battery", &on_battery,
 		      "lid-is-closed", &lid_is_closed,
 		      "lid-is-present", &lid_is_present,
-		      "is-docked", &is_docked,
 		      NULL);
 
 	g_print ("  daemon-version:  %s\n", daemon_version);
 	g_print ("  on-battery:      %s\n", on_battery ? "yes" : "no");
 	g_print ("  lid-is-closed:   %s\n", lid_is_closed ? "yes" : "no");
 	g_print ("  lid-is-present:  %s\n", lid_is_present ? "yes" : "no");
-	g_print ("  is-docked:       %s\n", is_docked ? "yes" : "no");
 	action = up_client_get_critical_action (client);
 	g_print ("  critical-action: %s\n", action);
 	g_free (action);
